/*
** Class:  LotROLinux.SettingsWindow
** Author: Alan Jackson
** Date:   6th June 2007
**
** Settings windows for the linux based launcher
** for the game Lord of the Rings Online
**
** Based on a script by SNy <SNy@bmx-chemnitz.de>
**
** (C) 2007-2008 AJackson <ajackson@bcs.org.uk>
**
** This file is part of LotROLinux.
**
** LotROLinux is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 3 of the License, or
** (at your option) any later version.
**
** LotROLinux is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with LotROLinux.  If not, see <http://www.gnu.org/licenses/>.
*/
using System;
using System.IO;
using Glade;
using Gtk;

namespace LotROLinux
{
	public class SettingsWindow
	{
		[Widget]
		private Dialog winSettings;

		[Widget]
        private ComboBox cboApp;
        
		[Widget]
        private ComboBox cboHiRes;
        
		[Widget]
        private ComboBox cboWineLog;
        
		[Widget]
        private FileChooserButton fcbGameDir;
        
		[Widget]
        private Entry txtDebug;
        
		[Widget]
        private Entry txtParams;
        
		[Widget]
        private Entry txtPrefix;

		[Widget]
        private ComboBox cboPrefix;
        
		[Widget]
        private Label lblPrefix;
        
		[Widget]
        private Entry txtProg;
        
		[Widget]
        private Entry txtPatchClient;

		[Widget]
        private Label lblPatchClient;

		private const string keyAppWine = "Wine";
		private const string keyAppCXGames = "CXGames";
		private const string keyAppCXOffice = "CXOffice";

		private DetermineOS osType;
		private string wineApp;
		private string wineProgram;
		private string wineParams;
		private string wineDebug;
		private string winePrefix;
		private string patchClient;
		private string gameDir;
		private string homeDir;
		private string baseDir;
		private bool wineLog;
		private bool hiRes;

		public SettingsWindow(string wProg,
			string wParams, string wDebug,
			string wPrefix, string pClient, bool wLog, string gDir, 
            bool hiResEnabled, bool usingDND,
			string icoFile, string wApp, string hDir,
			DetermineOS osType)
		{
			this.osType = osType;
			wineApp = wApp;
			wineLog = wLog;
			wineProgram = wProg;
			wineParams = wParams;
			wineDebug = wDebug;
			winePrefix = wPrefix;
			patchClient = pClient;
			gameDir = gDir;
			homeDir = hDir;
			hiRes = hiResEnabled;
			baseDir = "";

			Glade.XML gxml = new Glade.XML (null, "SettingsWindow.glade", "winSettings", null);
			gxml.Autoconnect (this);

			if (icoFile != null)
			{
				winSettings.Icon = Gdk.Pixbuf.LoadFromResource(icoFile);
			}

			txtProg.Text = wineProgram;
			txtParams.Text = wineParams;
			txtDebug.Text = wineDebug;
			txtPrefix.Text = winePrefix;
			fcbGameDir.SetCurrentFolder(gameDir);

			// Ensure fcbGameDir is set up
			while (Gtk.Application.EventsPending ())
				Gtk.Application.RunIteration();

			txtProg.HasFocus = true;
			
			if (usingDND)
			{
				lblPatchClient.Visible = true;
				txtPatchClient.Visible = true;
			} else {
				lblPatchClient.Visible = false;
				txtPatchClient.Visible = false;
			}
			txtPatchClient.Text = patchClient;

			cboApp.Clear();
	        CellRendererText cellApp = new CellRendererText();
	        cboApp.PackStart(cellApp, false);
	        cboApp.AddAttribute(cellApp, "text", 0);
	        ListStore storeApp = new ListStore(typeof (string));
	        cboApp.Model = storeApp;

            cboApp.AppendText("Wine");
            cboApp.AppendText("Crossover Games");
            cboApp.AppendText("Crossover Office");

			switch (wineApp)
			{
				case keyAppWine:
					cboApp.Active = 0;
					lblPrefix.Text = "WINEPREFIX";
					txtPrefix.Visible = true;
		            cboPrefix.Visible = false;
					break;
				case keyAppCXGames:
					cboApp.Active = 1;
					lblPrefix.Text = "Bottle Name";
					txtPrefix.Visible = false;
		            cboPrefix.Visible = true;
					break;
				case keyAppCXOffice:
					cboApp.Active = 2;
					lblPrefix.Text = "Bottle Name";
					txtPrefix.Visible = false;
		            cboPrefix.Visible = true;
					break;
			}

			cboHiRes.Clear();
	        CellRendererText cellHiRes = new CellRendererText();
	        cboHiRes.PackStart(cellHiRes, false);
	        cboHiRes.AddAttribute(cellHiRes, "text", 0);
	        ListStore storeHiRes = new ListStore(typeof (string));
	        cboHiRes.Model = storeHiRes;

            cboHiRes.AppendText("Enabled");
            cboHiRes.AppendText("Disabled");

			if (hiRes)
			{
				cboHiRes.Active = 0;
			} else {              
				cboHiRes.Active = 1;
			}

			cboWineLog.Clear();
	        CellRendererText cellWineLog = new CellRendererText();
	        cboWineLog.PackStart(cellWineLog, false);
	        cboWineLog.AddAttribute(cellWineLog, "text", 0);
	        ListStore storeWineLog = new ListStore(typeof (string));
	        cboWineLog.Model = storeWineLog;

            cboWineLog.AppendText("Yes");
            cboWineLog.AppendText("No");

			if (wLog)
			{
				cboWineLog.Active = 0;
			} else {              
				cboWineLog.Active = 1;
			}

			if (txtProg.Text != "")
			{
				txtProg.SelectRegion(txtProg.Text.Length,
					txtProg.Text.Length);
			}
		}

		public ResponseType Run()
		{
			return (ResponseType)winSettings.Run();
		}

		public void Destroy()
		{
			winSettings.Destroy();
		}

		public bool GetWineLog()
		{
			return wineLog;
		}
		
		public string GetWineProgram()
		{
			return wineProgram;
		}
		
		public string GetWineParams()
		{
			return wineParams;
		}

		public string GetWineDebug()
		{
			return wineDebug;
		}

		public string GetWinePrefix()
		{
			return winePrefix;
		}

		public string GetPatchClient()
		{
			return patchClient;
		}

		public string GetGameDir()
		{
			return gameDir;
		}

		public string GetApp()
		{
			return wineApp;
		}

		public bool GetHiRes()
		{
			return hiRes;
		}

		private void FindBottles()
		{
			DirectoryInfo bottleDir;

			cboPrefix.Clear();
	        CellRendererText cellPrefix = new CellRendererText();
	        cboPrefix.PackStart(cellPrefix, false);
	        cboPrefix.AddAttribute(cellPrefix, "text", 0);
	        ListStore storePrefix = new ListStore(typeof (string));
	        cboPrefix.Model = storePrefix;

			if (cboApp.Active == 1)
			{
				baseDir = String.Format("{0}{1}", homeDir, osType.SettingsCXG);

				bottleDir = new DirectoryInfo(baseDir);

				if (bottleDir.Exists)
				{
					ListBottles();
				}
			} else {
				baseDir = String.Format("{0}{1}", homeDir, osType.SettingsCXO);

				bottleDir = new DirectoryInfo(baseDir);

				if (bottleDir.Exists)
				{
					ListBottles();
				}
			}
		}

		private void ListBottles()
		{
			int currBottle = 0;
			DirectoryInfo dirs = new DirectoryInfo(baseDir);

			foreach(DirectoryInfo dir in dirs.GetDirectories())
			{
				DirectoryInfo bottle = new DirectoryInfo(String.Format("{0}/{1}/drive_c", baseDir, dir.Name));

				if (bottle.Exists)
				{
					cboPrefix.AppendText(dir.Name);
					if (dir.Name == winePrefix)
					{
						cboPrefix.Active = currBottle;
					}

					currBottle++;
				}
			}

			if (cboPrefix.Active == -1)
			{
				cboPrefix.Active = 0;
			}
		}

		protected virtual void OnCboAppChanged(object sender, System.EventArgs e)
		{
			if (cboApp.Active == 0)
			{
				lblPrefix.Text = "WINEPREFIX";
				txtPrefix.Visible = true;
                cboPrefix.Visible = false;
			} else {
				lblPrefix.Text = "Bottle Name";
				txtPrefix.Visible = false;
                cboPrefix.Visible = true;
				FindBottles();
			}
		}

		protected virtual void OnCboPrefixChanged(object sender, System.EventArgs e)
		{
			if (!fcbGameDir.Filename.StartsWith(String.Format("{0}/{1}", baseDir, cboPrefix.ActiveText)))
			{
				fcbGameDir.SetCurrentFolder(String.Format("{0}/{1}/drive_c/Program Files", baseDir, cboPrefix.ActiveText));
			}
		}

		protected virtual void OnBtnOkClicked(object sender, System.EventArgs e)
		{
			wineProgram = txtProg.Text;
			wineParams = txtParams.Text;
			winePrefix = txtPrefix.Text;
			wineDebug = txtDebug.Text;
			gameDir = fcbGameDir.Filename;
			patchClient = txtPatchClient.Text;

			switch (cboApp.Active)
			{
				case 0:
					wineApp = keyAppWine;
					winePrefix = txtPrefix.Text;
					break;
				case 1:
					wineApp = keyAppCXGames;
					winePrefix = cboPrefix.ActiveText;
					break;
				case 2:
					wineApp = keyAppCXOffice;
					winePrefix = cboPrefix.ActiveText;
					break;
			}

			if (cboHiRes.Active == 0)
			{
				hiRes = true;
			} else {
				hiRes = false;
			}

			if (cboWineLog.Active == 0)
			{
				wineLog = true;
			} else {
				wineLog = false;
			}
		}
	}
}
